package org.antlr.tool;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.util.List;
import java.util.Set;
import org.antlr.analysis.DFAState;
import org.antlr.analysis.DecisionProbe;
import org.antlr.analysis.Label;
import org.antlr.analysis.NFAState;
import org.antlr.tool.GrammarAST;
import org.antlr.tool.Message;
import org.stringtemplate.v4.ST;

public class GrammarNonDeterminismMessage extends Message {
  public DecisionProbe probe;
  
  public DFAState problemState;
  
  public GrammarNonDeterminismMessage(DecisionProbe probe, DFAState problemState) {
    super(200);
    this.probe = probe;
    this.problemState = problemState;
    if (probe.dfa.isTokensRuleDecision())
      setMessageID(209); 
  }
  
  public String toString() {
    GrammarAST decisionASTNode = this.probe.dfa.getDecisionASTNode();
    this.line = decisionASTNode.getLine();
    this.column = decisionASTNode.getCharPositionInLine();
    String fileName = this.probe.dfa.nfa.grammar.getFileName();
    if (fileName != null)
      this.file = fileName; 
    ST st = getMessageTemplate();
    List<Label> labels = this.probe.getSampleNonDeterministicInputSequence(this.problemState);
    String input = this.probe.getInputSequenceDisplay(labels);
    st.add("input", input);
    if (this.probe.dfa.isTokensRuleDecision()) {
      Set<Integer> disabledAlts = this.probe.getDisabledAlternatives(this.problemState);
      for (Integer altI : disabledAlts) {
        String tokenName = this.probe.getTokenNameForTokensRuleAlt(altI.intValue());
        NFAState ruleStart = this.probe.dfa.nfa.grammar.getRuleStartState(tokenName);
        this.line = ruleStart.associatedASTNode.getLine();
        this.column = ruleStart.associatedASTNode.getCharPositionInLine();
        st.add("disabled", tokenName);
      } 
    } else {
      st.add("disabled", this.probe.getDisabledAlternatives(this.problemState));
    } 
    List<Integer> nondetAlts = this.probe.getNonDeterministicAltsForState(this.problemState);
    NFAState nfaStart = this.probe.dfa.getNFADecisionStartState();
    int firstAlt = 0;
    if (nondetAlts != null)
      for (Integer displayAltI : nondetAlts) {
        if (DecisionProbe.verbose) {
          int tracePathAlt = nfaStart.translateDisplayAltToWalkAlt(displayAltI.intValue());
          if (firstAlt == 0)
            firstAlt = tracePathAlt; 
          List<? extends NFAState> path = this.probe.getNFAPathStatesForAlt(firstAlt, tracePathAlt, labels);
          st.addAggr("paths.{alt, states}", new Object[] { displayAltI, path });
          continue;
        } 
        if (this.probe.dfa.isTokensRuleDecision()) {
          String tokenName = this.probe.getTokenNameForTokensRuleAlt(displayAltI.intValue());
          st.add("conflictingTokens", tokenName);
          continue;
        } 
        st.add("conflictingAlts", displayAltI);
      }  
    st.add("hasPredicateBlockedByAction", Boolean.valueOf(this.problemState.dfa.hasPredicateBlockedByAction));
    return toString(st);
  }
}
